@@ -1,6 +1,6 @@
 use std::future::Future;
 
-use crate::{Endpoint, Request};
+use crate::{Endpoint, Request, Result};
 
 /// Endpoint for the [`before`](super::EndpointExt::before) method.
 pub struct Before<E, F> {
@@ -20,11 +20,11 @@ impl<E, F, Fut> Endpoint for Before<E, F>
 where
     E: Endpoint,
     F: Fn(Request) -> Fut + Send + Sync,
-    Fut: Future<Output = Request> + Send,
+    Fut: Future<Output = Result<Request>> + Send,
 {
     type Output = E::Output;
 
-    async fn call(&self, req: Request) -> Self::Output {
-        self.inner.call((self.f)(req).await).await
+    async fn call(&self, req: Request) -> Result<Self::Output> {
+        self.inner.call((self.f)(req).await?).await
     }
 }
