@@ -11,7 +11,7 @@ use crate::{
         Extensions, StatusCode, Version,
     },
     web::headers::Header,
-    Body, Error,
+    Body,
 };
 
 /// Component parts of an HTTP Response.
@@ -71,12 +71,6 @@ impl From<StatusCode> for Response {
     }
 }
 
-impl From<Error> for Response {
-    fn from(err: Error) -> Self {
-        err.as_response()
-    }
-}
-
 impl<T: Into<Body>> From<(StatusCode, T)> for Response {
     fn from((status, body): (StatusCode, T)) -> Self {
         Response::builder().status(status).body(body.into())
@@ -342,10 +336,6 @@ mod tests {
         assert_eq!(resp.status(), StatusCode::BAD_GATEWAY);
         assert!(resp.body.into_string().await.unwrap().is_empty());
 
-        let resp = Response::from(Error::new(StatusCode::BAD_GATEWAY).with_reason("bad gateway"));
-        assert_eq!(resp.status(), StatusCode::BAD_GATEWAY);
-        assert_eq!(resp.body.into_string().await.unwrap(), "bad gateway");
-
         let resp = Response::from((StatusCode::BAD_GATEWAY, Body::from("abc")));
         assert_eq!(resp.status(), StatusCode::BAD_GATEWAY);
         assert_eq!(resp.body.into_string().await.unwrap(), "abc");
